@@ -310,7 +310,7 @@ class ORM:
         self.__field_definitions__ = {}
         for key in dir(self):
             if not key.startswith('__') and not key.startswith('_') and key not in ['save', 'load', 'delete', 'to_json',
-                                                                                    'to_dict', 'create_table', 'select']:
+                                                                                    'to_dict', 'create_table', 'select', 'fetch', 'fetch_one']:
                 self.__field_definitions__[key] = getattr(self, key)
                 counter += 1
 
@@ -411,7 +411,7 @@ class ORM:
                     if new_id is not None:
                         current_value.value = new_id
                     else:
-                       current_value.value = current_value.default_value
+                        current_value.value = current_value.default_value
 
                 data[key] = current_value.value
             elif isinstance(value, IntegerField):
@@ -452,7 +452,7 @@ class ORM:
         """
         self.__dba__.create_table(self.__table_name__, True)
 
-    def _build_sql(self, column_names="*", join="", filter="", group_by="", having="", order_by=""):
+    def __build_sql(self, column_names="*", join="", filter="", group_by="", having="", order_by=""):
         """
         Helper method to build the SQL query
         :param column_names:
@@ -495,7 +495,7 @@ class ORM:
         :param order_by:
         :return:
         """
-        sql = self._build_sql(column_names, join, filter, group_by, having, order_by)
+        sql = self.__build_sql(column_names, join, filter, group_by, having, order_by)
         return self.__dba__.fetch_one(sql, params=params)
 
     def fetch(self, column_names="*", filter="", params=[], join="", group_by="", having="", order_by="", limit=10, skip=0):
@@ -512,7 +512,7 @@ class ORM:
         :param skip:
         :return:
         """
-        sql = self._build_sql(column_names, join, filter, group_by, having, order_by)
+        sql = self.__build_sql(column_names, join, filter, group_by, having, order_by)
         return self.__dba__.fetch(sql, params=params, limit=limit, skip=skip)
 
     def select (self, column_names="*", filter="", params=[], join="", group_by="", having="", order_by="", limit=10, skip=0):
@@ -584,8 +584,11 @@ class ORM:
             counter += 1
 
         try:
-
             record = self.__dba__.fetch_one(sql, input_params)
+            for key, value in data.items():
+                if key in self.__field_definitions__:
+                    if type(value) == BlobField and (isinstance(value, dict) or isinstance(value, list)):
+                        data[key] = json.dumps(value)
 
             if record["count_records"] == 0:
                 result = self.__dba__.insert(self.__table_name__, data)
