@@ -23,7 +23,7 @@ from tina4_python.Template import Template
 
 def is_int(v):
     try:
-        f = int(v)
+        int(v)
     except ValueError:
         return False
     return True
@@ -48,27 +48,32 @@ class Webserver:
                 for data in content_data:
                     data = data.split("=", 1)
                     body[data[0]] = unquote_plus(data[1])
-                return body
+                return body, {}
             elif self.lowercase_headers["content-type"] == "application/json":
                 # print("CONTENT", content, self.request)
                 try:
-                    return json.loads(content)
-                except:
-                    return content.decode("utf-8")
+                    return json.loads(content), {}
+                except Exception:
+                    return content.decode("utf-8"), {}
+
             elif self.lowercase_headers["content-type"] == "text/plain":
-                return content.decode("utf-8")
+                return content.decode("utf-8"), {}
             else:
                 content_data = self.lowercase_headers["content-type"].split("; ")
+
                 if content_data[0] == "multipart/form-data":
                     boundary = content_data[1].split("=")[1] + "\r\n"
                     content = b"\r\n" + content
                     data_array = content.split(str.encode(boundary))
                     body = {}
+                    files = {}
                     for data in data_array:
                         data = data.split(b"\r\n\r\n")
                         data_names = data[0].decode("utf-8").split("; ")
+
                         if data_names[0] == "Content-Disposition: form-data":
                             key_name = data_names[1].split("=")[1][1:-1]
+
                             if len(data_names) == 2:
                                 data_value = data[1].split(b"\r\n")[0]
                                 body[key_name] = unquote_plus(data_value.decode("utf-8"))
@@ -91,12 +96,25 @@ class Webserver:
                                 if "Content-Type" in meta_data:
                                     content_type = meta_data["Content-Type"]
 
-                                body[key_name] = {"file_name": file_name, "content_type": content_type,
-                                                  "content": base64.encodebytes(data_value).decode("utf-8").replace(
-                                                      "\n", "")}
-                    return body
+                                if key_name in body:
+                                    body[key_name] = [body[key_name]]
+                                    body[key_name].append({"file_name": file_name, "content_type": content_type,"content": base64.encodebytes(data_value).decode("utf-8").replace(
+                                        "\n", "")})
+                                else:
+                                    body[key_name] = {"file_name": file_name, "content_type": content_type,"content": base64.encodebytes(data_value).decode("utf-8").replace(
+                                        "\n", "")}
 
-        return {"data": base64.encodebytes(content).decode("utf-8").replace("\n", "")}
+                                if key_name in files:
+                                    files[key_name] = [files[key_name]]
+                                    files[key_name].append({"file_name": file_name, "content_type": content_type,"content": base64.encodebytes(data_value).decode("utf-8").replace(
+                                        "\n", "")})
+                                else:
+                                    files[key_name] = {"file_name": file_name, "content_type": content_type,"content": base64.encodebytes(data_value).decode("utf-8").replace(
+                                        "\n", "")}
+
+                    return body, files
+
+        return {"data": base64.encodebytes(content).decode("utf-8").replace("\n", "")}, {}
 
     async def send_basic_headers(self, headers):
         self.send_header("Access-Control-Allow-Origin", "*", headers)
@@ -181,11 +199,12 @@ class Webserver:
 
         content_length = await self.get_content_length()
         if method != Constant.TINA4_GET:
-            body = await self.get_content_body(content_length)
+            body, files = await self.get_content_body(content_length)
         else:
             body = None
+            files = None
 
-        request = {"params": params, "body": body, "raw_data": self.request, "url": self.path, "session": self.session,
+        request = {"params": params, "body": body, "files": files, "raw_data": self.request, "url": self.path, "session": self.session,
                    "headers": self.lowercase_headers, "raw_request": self.request_raw, "raw_content": self.content_raw,
                    "transport": transport, "asgi_response": asgi_response}
 
@@ -237,7 +256,7 @@ class Webserver:
     async def get_data(self, reader):
         try:
             raw_data = await reader.readuntil(b"\r\n\r\n")
-        except:
+        except Exception:
             raw_data = await reader.read(128)
 
         protocol = raw_data.decode("utf-8").split("\r\n", 1)[0]
@@ -267,7 +286,7 @@ class Webserver:
                     break
             try:
                 content = content_data.decode("utf-8")
-            except:  # probably binary or multipart form?
+            except Exception:  # probably binary or multipart form?
                 content = content_data
 
         return protocol, headers, lowercase_headers, content, raw_data, content_data
@@ -318,7 +337,8 @@ class Webserver:
                         headers = await self.get_headers(headers, self.response_protocol, HTTP_OK)
                         writer.write(headers + content)
                         await writer.drain()
-                        await writer.close()
+                        if writer is not None:
+                            writer.close()
                         request_handled = True
 
             if not request_handled:
@@ -353,6 +373,7 @@ class Webserver:
                             await writer.drain()
                         writer.close()
                     except BrokenPipeError as e:
+                        Debug.warning("Socket connection broken: " + str(e))
                         # socket got terminated
                         pass
                 else:
